import os
import sys
import json
import logging
import click
from pathlib import Path
from future.moves import subprocess

from mldock.config.config_manager import \
    ResourceConfigManager, SagifyConfigManager, PackageConfigManager, \
        HyperparameterConfigManager, InputDataConfigManager
from mldock.api.local import \
    _copy_boilerplate_to_dst, _rename_file

click.disable_unicode_literals_warning = True
logger=logging.getLogger('mlmldockdoc')

@click.command()
@click.option('--dir', help='Set the working directory for your sagify container.', required=True)
@click.option('--new', is_flag=True)
@click.option('--testing_framework', default=None, help='(Optional) Pytest framework. This creates a few health-check tests')
@click.pass_obj
def init(obj, dir, new, testing_framework):
    """
    Command to initialize container configs required by sagemaker-training.
    """
    helper_library_path=obj['helper_library_path']
    try:
        logger.info("Getting Sagify config")
        sagify_manager = SagifyConfigManager(
            filepath=os.path.join(dir, ".sagify.json")
        )
        if new:
            sagify_manager.setup_config()
            # write .sagify file
            logger.info("\n\nWriting a new .sagify file")
            sagify_manager.write_file()

        sagify_manager.pretty_print()

        logger.info("Getting Requirements")
        package_manager = PackageConfigManager(
            filepath=os.path.join(dir, "requirements.txt")
        )
        package_manager.pretty_print()
        # write to package manager
        package_manager.write_file()


        path_to_payload = Path(os.path.join(dir, "payload.json"))
        if not path_to_payload.exists():
            path_to_payload.write_text(json.dumps({"feature1": 10, "feature2":"groupA"}))

        # get sagify_module_path name
        sagify_config = sagify_manager.get_config()
        src_directory = os.path.join(
            dir,
            sagify_config.get("sagify_module_dir", "src")
        )

        template_dir = os.path.join(
            helper_library_path,
            'templates',
            sagify_config['platform']
        )

        sagify_module_path = os.path.join(
            src_directory,
            'container'
        )

        

        test_path = os.path.join(sagify_module_path, 'local_test', 'test_dir')
        config_path = os.path.join(test_path, 'input/config')

        if new:
            logger.info("Creating new workspace")
            _copy_boilerplate_to_dst(os.path.join(template_dir,'src/'), src_directory)
        if testing_framework == 'pytest':
            logger.info("Adding pytest container tests")
            _copy_boilerplate_to_dst(os.path.join(template_dir,'tests/'), 'tests/')
            logger.info("renaming test file")

            _rename_file(
                base_path='tests/container_health',
                current_filename='_template_test_container.py',
                new_filename='test_{ASSET_DIR}.py'.format(ASSET_DIR=dir)
            ) 

        # set resource config
        logger.info("\n\nGetting Container Resource config")
        resource_config = ResourceConfigManager(
            filepath=os.path.join(config_path, "resourceconfig.json")
        )
        resource_config.ask_for_resourceconfig()
        resource_config.write_file()
        # set input data channels
        logger.info("\n\nGetting Input Data config")
        input_data_channels = InputDataConfigManager(
            filepath=os.path.join(config_path, "inputdataconfig.json")
        )
        input_data_channels.ask_for_input_data_channels()
        input_data_channels.write_file()

        # set hyperparameters
        logger.info("\n\nGetting Hyperparameters config")
        hyperparameters = HyperparameterConfigManager(
            filepath=os.path.join(config_path, "hyperparameters.json")
        )
        hyperparameters.pretty_print()
        hyperparameters.ask_for_hyperparameters()
        hyperparameters.write_file()

        logger.info("\nlocal container volume is ready! ヽ(´▽`)/")
    except subprocess.CalledProcessError as e:
        logger.debug(e.output)
        raise
    except Exception as e:
        logger.info("{}".format(e))
        sys.exit(-1)



import os
import json
import click
import logging
from pathlib import Path

logger=logging.getLogger('mldock')

class BaseConfigManager:
    """Base config manager with basic read, write and update functionality.
    """
    def __init__(self, filepath):
        self.filepath = filepath
        if not self.file_exists(self.filepath):
            prompt_input = click.prompt("File not found. Create?", default="yes")
        
            if prompt_input:
                self.touch(self.filepath)

        self.config = self.load_config(self.filepath)

    @staticmethod
    def touch(path: str):
        """create a blank json file, seeded with {}

        Args:
            path (str): path to file
        """
        with open(path, 'a') as file_:
            json.dump({}, file_)

    @staticmethod
    def file_exists(filename: str) -> bool:
        """Check if file exists

        Args:
            filename (str): path to file

        Returns:
            bool: whether file exists
        """
        return os.path.exists(filename)

    @staticmethod
    def load_config(filename: str) -> dict:
        """loads config from file

        Args:
            filename (str): path to config to load
        Returns:
            dict: config
        """
        with open(filename, "r") as file_:
            config = json.load(file_)
            return config

    def pretty_print(self):
        """pretty prints a json config to terminal
        """
        pretty_config = json.dumps(self.config, indent=4, separators=(',', ': '), sort_keys=True)
        logger.info("{}\n".format(pretty_config))

    def write_file(self):
        """
        Trains ML model(s) locally
        :param dir: [str], source root directory
        :param docker_tag: [str], the Docker tag for the image
        :param image_name: [str], The name of the Docker image
        """

        with open(self.filepath, 'w') as config_file:
            json.dump(self.config, config_file, indent=4)

class ResourceConfigManager(BaseConfigManager):
    """Resource Config Manager for sagify
    """

    @staticmethod
    def ask_for_current_host_name():
        """prompt user for current host name

        Returns:
            return: current host name
        """
        current_host_name = chosen_python_index = click.prompt(
            text="Set current host name: ",
            default="algo",
        )

        return current_host_name

    @staticmethod
    def ask_for_network_interface_name():
        """prompt user for network interface name

        Returns:
            str: network interface name
        """
        network_interface_name = click.prompt(
            text="Set current host name: ",
            default="eth1",
        )

        return network_interface_name

    def ask_for_resourceconfig(self):
        """prompt user for resource config
        """
        current_host_name = self.ask_for_current_host_name()
        hosts = [current_host_name+"-1"]
        network_interface_name = self.ask_for_network_interface_name()

        self.config = {
            "current_host": hosts[0],
            "hosts": hosts,
            "network_interface_name": network_interface_name
        }

class SagifyConfigManager(BaseConfigManager):
    """Hyperparameter Config Manager for sagify
    """

    def setup_config(self):

        self.ask_for_image_name()
        self.ask_for_platform_name()
        self.ask_for_sagify_module_dir()
        self.ask_for_container_dir_name()
        self.ask_for_requirements_file_name()
        self.ask_for_python_version()

    def ask_for_image_name(self):
        """prompt user for image name

        Returns:
            return: image name
        """
        image_name = click.prompt(
            text="Set your image name: ",
        )

        self.config.update({
            'image_name':image_name
        })
    
    def ask_for_platform_name(self):
        """prompt user for platform name

        Returns:
            return: platform name
        """
        platform_name = click.prompt(
            text="Set your platform name:",
            default='sagemaker'
        )

        self.config.update({
            'platform':platform_name
        })

    def ask_for_container_dir_name(self):
        """prompt user for platform name

        Returns:
            return: platform name
        """
        container_dir_name = click.prompt(
            text="Set your container dir name:",
            default='sagify_base'
        )

        self.config.update({
            'container_dir':container_dir_name
        })

    def ask_for_sagify_module_dir(self):
        """prompt user for image name

        Returns:
            return: image name
        """
        
        sagify_module_dir = click.prompt(
            text="Set sagify module dir: ",
            default='src'
        )

        self.config.update({
            'sagify_module_dir': sagify_module_dir
        })

    def ask_for_requirements_file_name(self):
        """prompt user for image name

        Returns:
            return: image name
        """
        requirements_file_name = click.prompt(
            text="Set full path to requirements: ",
            default='requirements.txt'
        )

        self.config.update({
            'requirements_dir':requirements_file_name
        })

    def ask_for_python_version(self):
        logger.info("Select Python interpreter:")
        logger.info('{}'.format('\n'.join(['1 - Python3', '2 - Python2'])))

        def _validate_python_option(input_value):
            if int(input_value) not in {1, 2}:
                raise BadParameter(
                    message="invalid choice: {}. (choose from 1, 2)".format(str(input_value))
                )

            return int(input_value)

        chosen_python_index = click.prompt(
            text="Choose from 1, 2",
            default=1,
            value_proc=lambda x: _validate_python_option(x)
        )

        python_version = '3.6' if chosen_python_index == 1 else '2.7'

        self.config.update({
            'python_version':python_version
        })

    def ask_for_development_container_suffix(self):
        """prompt user for platform name

        Returns:
            return: platform name
        """
        dev_container_suffix = click.prompt(
            text="Set your dev container suffix:",
            default='dev'
        )

        self.config.update({
            'dev_container_suffix':dev_container_suffix
        })

    def get_config(self) -> dict:
        """get config object

        Returns:
            dict: config
        """
        return self.config

class PackageConfigManager(BaseConfigManager):
    """Hyperparameter Config Manager for sagify
    """
    @staticmethod
    def touch(filename):
        """creat an empty txt file

        Args:
            filename (str): path to file
        """
        Path(filename).touch()

    @staticmethod
    def load_config(filename: str) -> list:
        """load config

        Args:
            filename (str): path to config file to load

        Returns:
            list: config
        """
        path = Path(filename)
        with path.open() as f: 
            config = f.readlines()
        
        for index, c_package in enumerate(config):
            if c_package.endswith("\n"):
                config[index] = config[index].split("\n")[0]
        return config

    def get_config(self) -> list:
        """get config object

        Returns:
            list: config
        """
        return self.config

    def seed_packages(self, packages: list):
        """seeds config with required package modules

        Args:
            packages (list): packages to add to config
        """
        for new_package in packages:
            for config_package in self.config:
                if new_package not in config_package:
                    self.config.append(new_package)

    def pretty_print(self):
        """pretty prints a list config to terminal
        """
        pretty_config = json.dumps(self.config, indent=4, separators=(',', ': '))
        logger.info("{}\n".format(pretty_config))

    def write_file(self):
        """
        write to file
        """
        config = set(self.config)
        config_txt = "\n".join(config) + "\n"
        print(config_txt)
        Path(self.filepath).write_text(config_txt)
    

class HyperparameterConfigManager(BaseConfigManager):
    """Hyperparameter Config Manager for sagify
    """
    @staticmethod
    def is_float(s):
        """ Returns True is string is a number. """
        try:
            float(s)
            return True
        except ValueError:
            return False
    def ask_for_hyperparameters(self):
        """prompt user for hyperparameters
        """
        while True:
            name_value_pair = click.prompt(
                text="Add a hyperparameter. (Expects name:value). Hit enter to continue.",
                default="end",
                show_default=False,
                type=str
            )
            if name_value_pair == "end":
                logger.info("\nUpdated hyperparameters = {}".format(self.config))
                break
            elif ":" in name_value_pair and not name_value_pair == "name:value":
                name, value = name_value_pair.split(":",1)
                if value.isdigit():
                    value = int(value)
                elif self.is_float(value):
                    value = float(value)
                logger.info("Adding {}={}".format(name, value))
                self.config.update({name: value})
            else:
                logger.info("Expected format as name:value. Skipping")

class InputDataConfigManager(BaseConfigManager):
    """InputData Config Manager for sagify
    """
    attribute_config = { 
            "TrainingInputMode": "File",
            "S3DistributionType": "FullyReplicated",
            "RecordWrapperType": "None"
    }
    def ask_for_input_data_channels(self):
        """prompt users for input data channels
        """
        while True:
            channel = click.prompt(
                text="Add a data channel. (Expects name). Hit enter to continue.",
                default="NULL",
                show_default=False
            )
            if channel == "NULL":
                break
            else:
                logger.info("Adding {} channel".format(channel))
                self.config.update({
                    channel: self.attribute_config
                })



import os
from google.cloud import storage
import json
from pathlib import Path
from future.moves import subprocess
import logging

from mldock.platform_helpers.gcp.storage import _check_if_cloud_scheme, download_input_assets

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(None)
logger.setLevel(logging.INFO)

class MLFlowGCSEnvironment: 

    def __init__(self, base_dir, experiment_id, run_id, gcs_artifact_store):
        self.base_dir = base_dir
        self.experiment_id = experiment_id
        self.run_id = run_id
        self.gcs_artifact_store=gcs_artifact_store

        logger.debug("Creating training directories")
        self._create_training_directories()

        logger.debug("=== Local Containter ===\n\nCurrent base_dir: {}".format(self.base_dir))
        logger.debug("Current run_base_dir: {}".format(self.run_base_dir))
        logger.debug("Current GCS Artifact Store: {}".format(self.gcs_artifact_store))
        logger.debug("Current model: {}".format(self.model_local_path))
        logger.debug("Current metrics: {}".format(self.metrics_local_path))
        logger.debug("\n=== \nAll Set up and Ready to Go!\n ===\n")
    
    def _create_training_directories(self):
        """Create the directory structure and files necessary for training under the base path.
        """
        Path(self.run_base_dir).mkdir(parents=True, exist_ok=True)
        Path(self.model_local_path).mkdir(parents=True, exist_ok=True)
        Path(self.metrics_local_path).mkdir(parents=True, exist_ok=True)
        Path(self.params_local_path).mkdir(parents=True, exist_ok=True)
        Path(self.tags_local_path).mkdir(parents=True, exist_ok=True)

    @property
    def run_base_dir(self):
        return os.path.join(self.base_dir, self.experiment_id, self.run_id)

    @property
    def artifact_store_path(self):
        return self.gcs_artifact_store

    @property
    def model_local_path(self):
        return os.path.join(self.run_base_dir,'artifacts/model')

    @property
    def metrics_local_path(self):
        return os.path.join(self.run_base_dir,'metrics')

    @property
    def params_local_path(self):
        return os.path.join(self.run_base_dir,'params')

    @property
    def tags_local_path(self):
        return os.path.join(self.run_base_dir,'tags')

def update_bash_env_vars(key, value):
    """Execute a bash export to set ENV VARS

    Args:
        key (str): ENV var key
        value (str): ENV var value
    """
    _FILE_DIR_PATH = os.path.dirname(os.path.realpath(__file__))
    local_train_script_path = os.path.join(_FILE_DIR_PATH, 'update_var.sh')
    output = subprocess.check_output(
        [
            "{}".format(local_train_script_path),
            key,
            value
        ]
    )

class Environment:

    def __init__(self, base_dir):
        self.base_dir = base_dir
        self._create_training_directories()

    def _create_training_directories(self):
        """Create the directory structure, if not exists
        """
        logger.debug("Creating a new training folder under {} .".format(self.base_dir))

        try:
            self.input_data_dir.mkdir(parents=True, exist_ok=True)
            self.model_dir.mkdir(parents=True, exist_ok=True)
            self.input_config_dir.mkdir(parents=True, exist_ok=True)
            self.output_data_dir.mkdir(parents=True, exist_ok=True)
        except Exception as exception:
            print(exception)
            raise


    @property
    def input_dir(self):
        return Path(self.base_dir, 'input')

    @property
    def input_data_dir(self):
        return Path(self.input_dir, 'data')

    @property
    def input_config_dir(self):
        return Path(self.input_dir, 'config')

    @property
    def model_dir(self):
        return Path(self.base_dir, 'model')
    
    @property
    def output_data_dir(self):
        return Path(self.base_dir, 'output')

class AIPlatformEnvironment(Environment):
    @staticmethod
    def get_all_environ_vars(contains):
        """Get all environ vars matching contains='<PREFIX>'"""
        sm_channels = []
        for key,value in os.environ.items():
            if contains in key:
                sm_channels.append({'channel': key, 'path': value})
        return sm_channels 

    def download_input_data_from_cloud_storage(self, path):
        """
            download only cloud storage artifacts
        """
        scheme = 'gs' 
        is_cloud_storage =  _check_if_cloud_scheme(url=path, scheme=scheme)
        if is_cloud_storage:
            download_input_assets(storage_dir_path=path, local_path=self.input_data_dir, scheme=scheme)
        else:
            Exception("No Cloud storage url was found. Must have gs:// schema")
        folder = Path(path).name
        local_path = Path(self.input_data_dir, folder)
        return local_path

    def setup_input_data(self):
        """ Iterates through environment variables and downloads artifacts from storage.
        """
        sm_channels = self.get_all_environ_vars(contains='SM_CHANNEL_')
        for channel in sm_channels:
            _ = self.download_input_data_from_cloud_storage(path=channel['path'])
    


import os
import json
import logging
from pathlib import Path
from distutils.dir_util import copy_tree
from future.moves import subprocess

logger=logging.getLogger('mldock')

def train_model(script_path, test_dir, docker_tag, image_name):
    """
    Trains ML model(s) locally
    :param dir: [str], source root directory
    :param docker_tag: [str], the Docker tag for the image
    :param image_name: [str], The name of the Docker image
    """
    local_train_script_path = script_path
    test_path = test_dir

    output = subprocess.check_output(
        [
            "{}".format(local_train_script_path),
            "{}".format(os.path.abspath(test_path)),
            docker_tag,
            image_name
        ]
    )
    logger.debug(output.decode('utf-8'))
    logger.info("\nTraining Complete! ヽ(´▽`)/")

def local_train(dir, docker_tag, image_name):
    """
    Trains ML model(s) locally
    :param dir: [str], source root directory
    :param docker_tag: [str], the Docker tag for the image
    :param image_name: [str], The name of the Docker image
    """
    sagify_module_path = os.path.join(dir, 'sagify_base')
    local_train_script_path = os.path.join(sagify_module_path, 'local_test', 'train_local.sh')
    test_path = os.path.join(sagify_module_path, 'local_test', 'test_dir')

    if not os.path.isdir(test_path):
        raise ValueError("This is not a sagify directory: {}".format(dir))

    output = subprocess.check_output(
        [
            "{}".format(local_train_script_path),
            "{}".format(os.path.abspath(test_path)),
            docker_tag,
            image_name
        ]
    )
    logger.debug(output.decode('utf-8'))
    logger.info("\nTraining Complete! ヽ(´▽`)/")


def deploy_model(script_path, test_dir, docker_tag, image_name):
    """
    Deploys ML models(s) locally
    :param dir: [str], source root directory
    :param docker_tag: [str], the Docker tag for the image
    :param image_name: [str], The name of the Docker image
    """
    local_deploy_script_path = script_path
    test_path = test_dir

    output = subprocess.check_output(
        [
            "{}".format(local_deploy_script_path),
            "{}".format(os.path.abspath(test_path)),
            docker_tag,
            image_name
        ]
    )
    logger.debug(output.decode('utf-8'))

def local_deploy(dir, docker_tag, image_name):
    """
    Deploys ML models(s) locally
    :param dir: [str], source root directory
    :param docker_tag: [str], the Docker tag for the image
    :param image_name: [str], The name of the Docker image
    """
    sagify_module_path = os.path.join(dir, 'sagify_base')
    local_deploy_script_path = os.path.join(sagify_module_path, 'local_test', 'deploy_local.sh')
    test_path = os.path.join(sagify_module_path, 'local_test', 'test_dir')

    if not os.path.isdir(test_path):
        raise ValueError("This is not a sagify directory: {}".format(dir))

    output = subprocess.check_output(
        [
            "{}".format(local_deploy_script_path),
            "{}".format(os.path.abspath(test_path)),
            docker_tag,
            image_name
        ]
    )
    logger.debug(output.decode('utf-8'))

def get_local_container_ids(dir, docker_tag, image_name):
    """
    Deploys ML models(s) locally
    :param dir: [str], source root directory
    :param docker_tag: [str], the Docker tag for the image
    :param image_name: [str], The name of the Docker image
    """
    sagify_module_path = os.path.join(dir, 'sagify_base')
    local_deploy_script_path = os.path.join(sagify_module_path, 'local_test', 'deploy_local.sh')
    test_path = os.path.join(sagify_module_path, 'local_test', 'test_dir')

    if not os.path.isdir(test_path):
        raise ValueError("This is not a sagify directory: {}".format(dir))

    output = subprocess.check_output(
        [
            "{}".format(local_deploy_script_path),
            "{}".format(os.path.abspath(test_path)),
            docker_tag,
            image_name
        ]
    )
    logger.debug(output.decode('utf-8'))

def local_deploy_detached(dir, docker_tag, image_name):
    """
    Deploys ML models(s) locally
    :param dir: [str], source root directory
    :param docker_tag: [str], the Docker tag for the image
    :param image_name: [str], The name of the Docker image
    """
    sagify_module_path = os.path.join(dir, 'sagify_base')
    local_deploy_script_path = os.path.join(sagify_module_path, 'local_test', 'deploy_local.sh')
    test_path = os.path.join(sagify_module_path, 'local_test', 'test_dir')

    if not os.path.isdir(test_path):
        raise ValueError("This is not a sagify directory: {}".format(dir))

    output = subprocess.Popen(
        [
            "{}".format(local_deploy_script_path),
            "{}".format(os.path.abspath(test_path)),
            docker_tag,
            image_name
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT
    )
    logger.debug(output.decode('utf-8'))
    logger.info("\nDeploy Ready! ヽ(´▽`)/")
    logger.info(
        "\nRemember to stop the container once you're done."
        "\n\nRun:\n>> docker ps\n\nfollowed by:\n>> docker stop <container-id>\n"
    )

def docker_build(
    script_path: str,
    image_name: str,
    dockerfile_path: str,
    module_path: str,
    target_dir_name: str,
    requirements_file_path: str,
    no_cache: bool
):
    """Runs the build executable from script path, passing a set of arguments in a command line subprocess.

    Args:
        script_path (str): relative path to script when run on root
        base_path (str):
        image_name (str):
        dockerfile_path (str):
        module_path (str):
        target_dir_name (str):
        requirements_file_path (str):
    """
    logger.info("\nStarting build...\n") 
    _ = subprocess.check_call([
                                script_path,
                                image_name,
                                dockerfile_path,
                                module_path,
                                target_dir_name,
                                requirements_file_path,
                                str(no_cache).lower()
    ])
    logger.info("\nBuild Complete! ヽ(´▽`)/")

def _rename_file(base_path, current_filename, new_filename):
    """renames filename for a given base_path, saving the file in the same base_path

    Args:
        base_path ([type]): directory path containing file to rename
        current_filename ([type]): current name of the file to rename
        new_filename ([type]): new name for the renamed file
    """
    Path(base_path, current_filename).rename(Path(base_path, new_filename))

def _create_empty_file(base_path, filename):
    """renames filename for a given base_path, saving the file in the same base_path

    Args:
        base_path ([type]): directory path containing file to rename
        current_filename ([type]): current name of the file to rename
        new_filename ([type]): new name for the renamed file
    """
    Path(base_path, filename).touch(exist_ok=True)

def _copy_boilerplate_to_dst(src: str, dst: str):
    """[summary]

    Args:
        src (str): [description]
        dst (str): [description]
    """
    source_path = str(Path(src).absolute())
    destination_path = str(Path(dst).absolute())
    copy_tree(source_path, destination_path)


def run_predict(script_path: str, host: str, payload: str, content_type: str):
    """Runs the predict shell script and passes the arguments as expected by the script.

    Args:
        script_path (str): relative path to script when run on root
        host (str): http/https host uri
        payload (str): relative path to payload file when run on root
        content_type (str): content type for request
    """
    logger.info("\nRunning Prediction...\n") 
    result = subprocess.run([
                                     script_path,
                                     host,
                                     payload,
                                     content_type
    ], stdout=subprocess.PIPE)

    if content_type == 'application/json':
        decoded_stdout = json.loads(result.stdout.decode())
        pretty_output = json.dumps(decoded_stdout, indent=4, separators=(',', ': '), sort_keys=True)
    else:
        pretty_output = result.stdout
    logger.info(pretty_output)
    logger.info("\nPrediction Complete! ヽ(´▽`)/")
    return pretty_output


import os
import sys
import json
import logging
import click
from pathlib import Path
from future.moves import subprocess

from mldock.config.config_manager import \
    ResourceConfigManager, SagifyConfigManager, PackageConfigManager, \
        HyperparameterConfigManager, InputDataConfigManager
from mldock.api.local import \
    run_predict, _copy_boilerplate_to_dst, docker_build, \
        local_train, local_deploy_detached, _rename_file

from mldock.api.local import \
    train_model, deploy_model

click.disable_unicode_literals_warning = True
logger=logging.getLogger('mldock')

@click.group()
def local():
    """
    Commands for local operations: train and deploy
    """
    pass

@click.command()
@click.option('--dir', help='Set the working directory for your sagify container.', required=True)
@click.option('--no-cache', help='builds container from scratch', is_flag=True)
@click.pass_obj
def build(obj, dir, no_cache):
    """build image using docker

    Args:
        dir (str): directory containing model assets
    """
    helper_library_path = obj['helper_library_path']
    try:
        sagify_manager = SagifyConfigManager(
            filepath=os.path.join(dir, ".sagify.json")
        )
        # get sagify_module_path name
        sagify_config = sagify_manager.get_config()
        image_name = sagify_config.get("image_name", None)
        container_dir = sagify_config.get("container_dir", None)
        module_path = os.path.join(
                    dir,
            sagify_config.get("sagify_module_dir", "src"),
        )
        dockerfile_path = os.path.join(
            dir,
            sagify_config.get("sagify_module_dir", "src"),
            container_dir
        )
        requirements_file_path = os.path.join(
            dir,
            sagify_config.get("requirements.txt", "requirements.txt")
        )
        if image_name is None:
            logger.info("\nimage_name cannot be None")
        elif image_name.endswith(":latest"):
            logger.info("\nImage version is not supported at this point. Please remove :latest versioning")
        else:
            docker_build(
                script_path=os.path.join(helper_library_path,'api/build.sh'),
                image_name=image_name,
                dockerfile_path=dockerfile_path,
                module_path=module_path,
                target_dir_name=sagify_config.get("sagify_module_dir", "src"),
                requirements_file_path=requirements_file_path,
                no_cache=no_cache
            )
    except subprocess.CalledProcessError as e:
        logger.debug(e.output)
        raise
    except Exception as e:
        logger.info("{}".format(e))
        sys.exit(-1)

@click.command()
@click.option('--payload', default=None, help='path to payload file', required=True)
@click.option('--content-type', help='format of payload', type=click.Choice(['json', 'csv'], case_sensitive=False))
@click.option('--host', help='host url at which model is served', type=str, default='http://127.0.0.1:8080')
@click.pass_obj
def predict(obj, payload, content_type, host):
    """
    Command to run curl predict against an host served on localhost:8080
    """
    helper_library_path = obj['helper_library_path']
    try:
        script_path=os.path.join(helper_library_path,'api/predict.sh')
        if payload is None:
            logger.info("\nPayload cannot be None")
        else:
            if content_type == "csv":
                content_type = 'text/csv'
                _ = run_predict(
                    script_path=script_path,
                    host=host,
                    payload=payload,
                    content_type=content_type
                )
            elif content_type == 'json':
                content_type = 'application/json'
                _ = run_predict(
                    script_path=script_path,
                    host=host,
                    payload=payload,
                    content_type=content_type
                )
            else:
                raise TypeError("content-type of payload no supported. Only csv and json are supported at the moment")
    except subprocess.CalledProcessError as e:
        logger.debug(e.output)
        raise
    except Exception as e:
        logger.info("{}".format(e))
        sys.exit(-1)

@click.command()
@click.option('--dir', help='Set the working directory for your sagify container.', required=True)
@click.option('--tag', help='docker tag', type=str, default='latest')
@click.pass_obj
def train(obj, dir, tag):
    """
    Command to train ML model(s) locally
    """
    sagify_manager = SagifyConfigManager(
        filepath=os.path.join(dir, ".sagify.json")
    )
    # get sagify_module_path name
    sagify_config = sagify_manager.get_config()
    module_path = os.path.join(
                dir,
        sagify_config.get("sagify_module_dir", "src"),
    )
    image_name = sagify_config.get("image_name", None)
    platform = sagify_config.get("platform", None)
    container_dir = sagify_config.get("container_dir", None)
    helper_library_path = obj['helper_library_path']
    try:
        if platform == "sagemaker":
            local_train(
                dir=module_path,
                docker_tag=tag,
                image_name=image_name
            )
        else:
            train_model(
                script_path=os.path.join(helper_library_path, "api", "train_model.sh"),
                test_dir=os.path.join(module_path, container_dir, "local_test/test_dir"),
                docker_tag=tag,
                image_name=image_name
            )

        logger.info("Local training completed successfully!")
    except ValueError:
        logger.info("This is not a sagify directory: {}".format(dir))
        sys.exit(-1)
    except subprocess.CalledProcessError as e:
        logger.debug(e.output)
        raise
    except Exception as e:
        logger.info("{}".format(e))
        sys.exit(-1)


@click.command()
@click.option('--dir', help='Set the working directory for your sagify container.', required=True)
@click.option('--tag', help='docker tag', type=str, default='latest')
@click.option('--host', help='host url at which model is served', type=str, default='http://127.0.0.1:8080')
@click.pass_obj
def deploy(obj, dir, tag, host):
    """
    Command to deploy ML model(s) locally
    """
    helper_library_path=obj['helper_library_path']
    sagify_manager = SagifyConfigManager(
        filepath=os.path.join(dir, ".sagify.json")
    )
    # get sagify_module_path name
    sagify_config = sagify_manager.get_config()
    module_path = os.path.join(
                dir,
        sagify_config.get("sagify_module_dir", "src"),
    )
    image_name = sagify_config.get("image_name", None)
    platform = sagify_config.get("platform", None)
    container_dir = sagify_config.get("container_dir", None)
    helper_library_path = obj['helper_library_path']
    try:
        logger.info("Started local deployment at {}...\n".format(host))
        if platform == "sagemaker":
        
            local_deploy_detached(
                dir=module_path,
                docker_tag=tag,
                image_name=image_name
            )
        else:
            deploy_model(
                script_path=os.path.join(helper_library_path, "api", "deploy_model.sh"),
                test_dir=os.path.join(module_path, container_dir, "local_test/test_dir"),
                docker_tag=tag,
                image_name=image_name
            )

    except ValueError:
        logger.info("This is not a sagify directory: {}".format(dir))
        sys.exit(-1)
    except subprocess.CalledProcessError as e:
        logger.debug(e.output)
        raise
    except Exception as e:
        logger.info("{}".format(e))
        sys.exit(-1)

local.add_command(build)
local.add_command(predict)
local.add_command(train)
local.add_command(deploy)



import logging
from pathlib import Path
from distutils.dir_util import copy_tree
from future.moves import subprocess

logger=logging.getLogger('mldock')

def push_to_ecr(script_path: str, base_path:str, image_name: str, dockerfile_path: str, module_path: str, target_dir_name: str, requirements_file_path: str):
    """Runs the predict shell script and passes the arguments as expected by the script.

    Args:
        script_path (str): relative path to script when run on root
        host (str): http/https host uri
        payload (str): relative path to payload file when run on root
        content_type (str): content type for request
    """
    logger.info("\nStarting Upload to ECR...\n") 
    _ = subprocess.check_call([
                                script_path,
                                image_name,
                                base_path,
                                dockerfile_path,
                                module_path,
                                target_dir_name,
                                requirements_file_path
    ])
    logger.info("\nUpload to ECR complete! ヽ(´▽`)/")



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the 'License'). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the 'license' file accompanying this file. This file is
# distributed on an 'AS IS' BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
"""This module contains utilities related to logging."""
from __future__ import absolute_import

import json
import logging

import sagemaker_training


def get_logger():
    """Return a logger with the name 'sagemaker-training-toolkit',
    creating it if necessary.
    """
    return logging.getLogger("sagemaker-training-toolkit")


def configure_logger(level, log_format="%(asctime)s %(name)-12s %(levelname)-8s %(message)s"):
    # type: (int, str) -> None
    """Set logger configuration.
    Args:
        level (int): Logger level.
        log_format (str): Logger format.
    """
    logging.basicConfig(format=log_format, level=level)

    if level >= logging.INFO:
        logging.getLogger("boto3").setLevel(level)
        logging.getLogger("s3transfer").setLevel(level)
        logging.getLogger("botocore").setLevel(level)


def log_script_invocation(cmd, env_vars, logger=None):
    """Log a message with level INFO including information on the user script invoked.
    Args:
        cmd (str): Command used to invoke the script.
        env_vars (dict): Environment variables.
        logger (logging.Logger): Logger used to log the message.
    """
    logger = logger or get_logger()

    prefix = "\n".join(["%s=%s" % (key, value) for key, value in env_vars.items()])
    env = sagemaker_training.environment.Environment()
    message = """Invoking user script
Training Env:
%s
Environment variables:
%s
Invoking script with the following command:
%s
""" % (
        json.dumps(dict(env), indent=4),
        prefix,
        " ".join(cmd),
    )
    logger.info(message)



"""Utilities to deploy a model to GCP services
"""

# Copyright 2019 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Deploy a model in AI Platform."""

import logging
import json
import time

from googleapiclient import discovery
from googleapiclient import errors

_WAIT_FOR_COMPLETION_SLEEP_SECONDS = 10
_PYTHON_VERSION = '3.5'
_RUN_TIME_VERSION = '1.15'


def _create_service():
    """Gets service instance to start API searches.
    :return:
    """
    return discovery.build('ml', 'v1')


class AIPlatformModel(object):
    def __init__(self, project_id):
        self._project_id = project_id
        self._service = _create_service()

    def model_exists(self, model_name):
        """
        :param model_name:
        :return:
        """
        models = self._service.projects().models()
        try:
            response = models.list(
                parent='projects/{}'.format(self._project_id)).execute()
            if response:
                for model in response['models']:
                    if model['name'].rsplit('/', 1)[1] == model_name:
                        return True
                    else:
                        return False
        except errors.HttpError as err:
            logging.error('%s', json.loads(err.content)['error']['message'])

    def _list_model_versions(self, model_name):
        """Lists existing model versions in the project.
        Args:
          model_name: Model name to list versions for.
        Returns:
          Dictionary of model versions.
        """
        versions = self._service.projects().models().versions()

        try:
            return versions.list(
                parent='projects/{}/models/{}'.format(self._project_id,
                                                      model_name)).execute()
        except errors.HttpError as err:
            logging.error('%s', json.loads(err.content)['error']['message'])

    def create_model(self, model_name, model_region='us-central1'):
        """
        :param model_name:
        :param model_region:
        :return:
        """
        if not self.model_exists(model_name):
            body = {
                'name': model_name,
                'regions': model_region,
                'description': 'MLflow model'
            }
            parent = 'projects/{}'.format(self._project_id)
            try:
                self._service.projects().models().create(
                    parent=parent, body=body).execute()
                logging.info('Model "%s" has been created.', model_name)
            except errors.HttpError as err:
                logging.error('"%s". Skipping model creation.',
                              json.loads(err.content)['error']['message'])
        else:
            logging.warning('Model "%s" already exists.', model_name)

    def deploy_model(self, bucket_name, model_name, model_version,
                     runtime_version=_RUN_TIME_VERSION):
        """Deploys model on AI Platform.
        Args:
          bucket_name: Cloud Storage Bucket name that stores saved model.
          model_name: Model name to deploy.
          model_version: Model version.
          runtime_version: Runtime version.
        Raises:
          RuntimeError if deployment completes with errors.
        """
        # For details on request body, refer to:
        # https://cloud.google.com/ml-engine/reference/rest/v1/projects
        # .models.versions/create
        model_version_exists = False
        model_versions_list = self._list_model_versions(model_name)
        #  Field: version.name Error: A name should start with a letter and
        #  contain only letters, numbers and underscores
        model_version = 'mlflow_{}'.format(model_version)

        if model_versions_list:
            for version in model_versions_list['versions']:
                if version['name'].rsplit('/', 1)[1] == model_version:
                    model_version_exists = True

        if not model_version_exists:
            request_body = {
                'name': model_version,
                'deploymentUri': '{}'.format(bucket_name),
                'framework': 'TENSORFLOW',
                'runtimeVersion': runtime_version,
                'pythonVersion': _PYTHON_VERSION
            }
            parent = 'projects/{}/models/{}'.format(self._project_id,
                                                    model_name)
            response = self._service.projects().models().versions().create(
                parent=parent, body=request_body).execute()
            op_name = response['name']
            while True:
                deploy_status = (
                    self._service.projects().operations().get(
                        name=op_name).execute())
                if deploy_status.get('done'):
                    logging.info('Model "%s" with version "%s" deployed.',
                                 model_name,
                                 model_version)
                    break
                if deploy_status.get('error'):
                    logging.error(deploy_status['error'])
                    raise RuntimeError(
                        'Failed to deploy model for serving: {}'.format(
                            deploy_status['error']))
                logging.info(
                    'Waiting for %d seconds for "%s" with "%s" version to be '
                    'deployed.',
                    _WAIT_FOR_COMPLETION_SLEEP_SECONDS, model_name,
                    model_version)
                time.sleep(_WAIT_FOR_COMPLETION_SLEEP_SECONDS)

        else:
            logging.info('Model "%s" with version "%s" already exists.',
                         model_name,
                         model_version)



import os
from mldock.platform_helpers.gcp import storage
from mldock.platform_helpers.gcp.environment import AIPlatformEnvironment as Environment

class TrainingContainer:
    """A set of tasks for setup and cleanup of container"""
    def startup(base_dir,logger, env='prod'):
        logger.info("\n\n --- Running Startup Script ---\n\nSetting Up Training Container")
        environment = Environment(base_dir=base_dir)
        if env=="prod":
            logger.info("Env == Prod")
            environment.setup_input_data()
        logger.info("\n\n --- Setup Complete --- \n\n")
        

    def cleanup(base_dir, logger, env='prod'):
        """clean up tasks executed on container task complete"""
        logger.info("\n\n --- Running Cleanup Script ---\n\nCleaning Up Training Container")
        if env=="prod":
            logger.info("Env == Prod")
            storage.package_and_upload_model_dir(local_path=os.path.join(base_dir, 'model'), storage_dir_path=os.environ['SM_MODEL_DIR'], scheme='gs')
            storage.package_and_upload_output_data_dir(local_path=os.path.join(base_dir, 'output'), storage_dir_path=os.environ['SM_OUTPUT_DATA_DIR'], scheme='gs')
        logger.info("\n\n --- Cleanup Complete --- \n\n")

class ServingContainer:
    """
        A set of tasks for setup and cleanup of container
    
        note:
            - Only supports a startup script. Cleanup is a  bit fuzzy for serving.
    """
    def startup(base_dir, logger, env='prod'):
        logger.info("\n\n --- Running Startup Script ---\n\nSetting Up Training Container")
        environment = Environment(base_dir=base_dir)
        if env=="prod":
            logger.info("Env == Prod")
            environment.setup_input_data()
        logger.info("\n\n --- Setup Complete --- \n\n")



import os
import logging
import click
from mldock.api.push import \
    push_to_ecr
from mldock.config.config_manager import \
    SagifyConfigManager

logger=logging.getLogger('mldock')

@click.command()
@click.option('--dir', help='Set the working directory for your sagify container.', required=True)
def push(dir):
    """
    Command to push docker container image to ECR using your default AWS sdk credentials.

    note: Override default AWS credentials by providing AWS_DEFAULT_PROFILE environment variable.
    """
    sagify_manager = SagifyConfigManager(
        filepath=os.path.join(dir, ".sagify.json")
    )
    # get sagify_module_path name
    sagify_config = sagify_manager.get_config()
    image_name = sagify_config.get("image_name", None)
    dockerfile_path = os.path.join(
        dir,
        sagify_config.get("sagify_module_dir", "src"),
        'sagify_base'
    )
    if image_name is None:
        logger.info("\nimage_name cannot be None")
    elif image_name.endswith(":latest"):
        logger.info("\nImage version is not supported at this point. Please remove :latest versioning")
    else:
        push_to_ecr(
            script_path='serum/api/build_and_push.sh',
            base_path=dir,
            image_name=image_name,
            dockerfile_path=dockerfile_path,
            module_path=sagify_config.get("sagify_module_dir", "src"),
            target_dir_name=sagify_config.get("sagify_module_dir", "src"),
            requirements_file_path=sagify_config.get("requirements.txt", "requirements.txt")
        )



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
"""This module contains string constants representing environment variables
and related parameters.
"""
from __future__ import absolute_import

SAGEMAKER_PREFIX = "sagemaker_"  # type: str
CURRENT_HOST_ENV = "CURRENT_HOST"  # type: str
USER_PROGRAM_PARAM = "sagemaker_program"  # type: str
USER_PROGRAM_ENV = USER_PROGRAM_PARAM.upper()  # type: str
S3_OUTPUT_LOCATION_PARAM = "sagemaker_s3_output"  # type: str
S3_OUTPUT_LOCATION_ENV = S3_OUTPUT_LOCATION_PARAM.upper()  # type: str
S3_ENDPOINT_URL = "S3_ENDPOINT_URL"  # type: str
TRAINING_JOB_ENV = "TRAINING_JOB_NAME"  # type: str
SUBMIT_DIR_PARAM = "sagemaker_submit_directory"  # type: str
SUBMIT_DIR_ENV = SUBMIT_DIR_PARAM.upper()  # type: str
ENABLE_METRICS_PARAM = "sagemaker_enable_cloudwatch_metrics"  # type: str
ENABLE_METRICS_ENV = ENABLE_METRICS_PARAM.upper()  # type: str
LOG_LEVEL_PARAM = "sagemaker_container_log_level"  # type: str
LOG_LEVEL_ENV = LOG_LEVEL_PARAM.upper()  # type: str
JOB_NAME_PARAM = "sagemaker_job_name"  # type: str
JOB_NAME_ENV = JOB_NAME_PARAM.upper()  # type: str
TUNING_METRIC_PARAM = "_tuning_objective_metric"  # type: str
DEFAULT_MODULE_NAME_PARAM = "default_user_module_name"  # type: str
MPI_ENABLED = "sagemaker_mpi_enabled"  # type: str
PARAMETER_SERVER_ENABLED = "sagemaker_parameter_server_enabled"  # type: str
REGION_NAME_PARAM = "sagemaker_region"  # type: str
REGION_NAME_ENV = REGION_NAME_PARAM.upper()  # type: str
DEFAULT_INVOCATIONS_ACCEPT_ENV = "SAGEMAKER_DEFAULT_INVOCATIONS_ACCEPT"  # type: str
SAGEMAKER_BIND_TO_PORT_ENV = "SAGEMAKER_BIND_TO_PORT"  # type: str
SAGEMAKER_SAFE_PORT_RANGE_ENV = "SAGEMAKER_SAFE_PORT_RANGE"  # type: str
FRAMEWORK_TRAINING_MODULE_ENV = "SAGEMAKER_TRAINING_MODULE"  # type: str
SAGEMAKER_HYPERPARAMETERS = (
    USER_PROGRAM_PARAM,
    SUBMIT_DIR_PARAM,
    ENABLE_METRICS_PARAM,
    REGION_NAME_PARAM,
    LOG_LEVEL_PARAM,
    JOB_NAME_PARAM,
    DEFAULT_MODULE_NAME_PARAM,
    TUNING_METRIC_PARAM,
    S3_OUTPUT_LOCATION_PARAM,
)  # type: tuple
MPI_PROCESSES_PER_HOST = "sagemaker_mpi_num_of_processes_per_host"  # type: int
MPI_NUM_PROCESSES = "sagemaker_mpi_num_processes"  # type: int
MPI_CUSTOM_OPTIONS = "sagemaker_mpi_custom_mpi_options"  # type: str
SAGEMAKER_NETWORK_INTERFACE_NAME = "sagemaker_network_interface_name"  # type: str



from locative_ml_helpers.platform_helpers.gcp.environment import Environment
import argparse

def str2bool(arg_value: str) -> bool:
    if isinstance(arg_value, bool):
        return arg_value
    if arg_value.lower() in ('yes', 'true', 't', 'y', '1'):
        return True
    elif arg_value.lower() in ('no', 'false', 'f', 'n', '0'):
        return False
    else:
        raise argparse.ArgumentTypeError('Boolean value expected.')

if __name__ == '__main__':

    PARSER= argparse.ArgumentParser()
    ## User set params which link to Sagemaker params
    PARSER.add_argument('--prod', type=str2bool,
                        default=False,
                        help='Expects a .csv file, representing the lookup.')
    ARGS, _ = PARSER.parse_known_args()
    print("Setting up Environment")
    env = Environment(base_dir='/opt/ml/')
    if ARGS.prod:
        env.setup_input_data()
    print("Done")



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="mldock", # Replace with your own username
    version="0.0.40",
    author="SheldonGrant",
    author_email="sheldz.shakes.williams@gmail.com",
    description="Global Machine learning helpers for docker based development. Build, train and deploy on cloud with docker",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/SheldonGrant/locative-ml-global-helpers",
    packages=setuptools.find_packages(where='.'),
    package_data={
        'mldock': [
            'api/*.sh',
            'platform_helpers/gcp/*.sh',
            'platform_helpers/gcp/__init__.py',
            'platform_helpers/gcp/environment.py',
            'platform_helpers/gcp/storage.py',
            'templates/sagemaker/tests/container_health/*.py',
            'templates/sagemaker/src/',
            'templates/sagemaker/src/*.py',
            'templates/sagemaker/src/utils',
            'templates/sagemaker/src/utils/*py',
            'templates/sagemaker/src/sagify_base',
            'templates/sagemaker/src/sagify_base/config.json',
            'templates/sagemaker/src/sagify_base/*.sh',
            'templates/sagemaker/src/sagify_base/Dockerfile',
            'templates/sagemaker/src/sagify_base/ __init__.py',
            'templates/sagemaker/src/sagify_base/training/__init__.py',
            'templates/sagemaker/src/sagify_base/training/train',
            'templates/sagemaker/src/sagify_base/prediction/*.py',
            'templates/sagemaker/src/sagify_base/prediction/serve',
            'templates/sagemaker/src/sagify_base/prediction/nginx.conf',
            'templates/sagemaker/src/sagify_base/local_test/*.sh',
            'templates/sagemaker/src/sagify_base/local_test/test_dir/output/.gitkeep',
            'templates/sagemaker/src/sagify_base/local_test/test_dir/model/.gitkeep',
            'templates/sagemaker/src/sagify_base/local_test/test_dir/input/config/*.json',
            'templates/sagemaker/src/sagify_base/local_test/test_dir/input/data/training/',
            'templates/aiplatform/tests/container_health/*.py',
            'templates/aiplatform/src/',
            'templates/aiplatform/src/prediction.py',
            'templates/aiplatform/src/trainer.py',
            'templates/aiplatform/src/sagify_base',
            'templates/aiplatform/src/sagify_base/config.json',
            'templates/aiplatform/src/sagify_base/*.sh',
            'templates/aiplatform/src/sagify_base/*.py',
            'templates/aiplatform/src/sagify_base/Dockerfile',
            'templates/aiplatform/src/sagify_base/ __init__.py',
            'templates/aiplatform/src/sagify_base/training/__init__.py',
            'templates/aiplatform/src/sagify_base/training/train',
            'templates/aiplatform/src/sagify_base/prediction/*.py',
            'templates/aiplatform/src/sagify_base/prediction/serve',
            'templates/aiplatform/src/sagify_base/prediction/nginx.conf',
            'templates/aiplatform/src/sagify_base/local_test/*.sh',
            'templates/aiplatform/src/sagify_base/local_test/test_dir/output/.gitkeep',
            'templates/aiplatform/src/sagify_base/local_test/test_dir/model/.gitkeep',
            'templates/aiplatform/src/sagify_base/local_test/test_dir/input/config/*.json',
            'templates/aiplatform/src/sagify_base/local_test/test_dir/input/data/training/',
            'templates/bare/tests/container_health/*.py',
            'templates/bare/service/docker-compose.yml',
            'templates/bare/src/',
            'templates/bare/src/env.py',
            'templates/bare/src/prediction.py',
            'templates/bare/src/trainer.py',
            'templates/bare/src/sagify_base',
            'templates/bare/src/sagify_base/config.json',
            'templates/bare/src/sagify_base/*.sh',
            'templates/bare/src/sagify_base/Dockerfile',
            'templates/bare/src/sagify_base/ __init__.py',
            'templates/bare/src/sagify_base/training/',
            'templates/bare/src/sagify_base/training/*.sh',
            'templates/bare/src/sagify_base/training/startup.py',
            'templates/bare/src/sagify_base/training/cleanup.py',
            'templates/bare/src/sagify_base/training/train.py',
            'templates/bare/src/sagify_base/prediction/',
            'templates/bare/src/sagify_base/prediction/serve.py',
            'templates/bare/src/sagify_base/prediction/startup.py',
            'templates/bare/src/sagify_base/prediction/wsgi.py',
            'templates/bare/src/sagify_base/prediction/predictor.py',
            'templates/bare/src/sagify_base/prediction/*.sh',
            'templates/bare/src/sagify_base/prediction/nginx.conf',
            'templates/bare/src/sagify_base/local_test/*.sh',
            'templates/bare/src/sagify_base/local_test/test_dir/output/.gitkeep',
            'templates/bare/src/sagify_base/local_test/test_dir/model/.gitkeep',
            'templates/bare/src/sagify_base/local_test/test_dir/input/config/*.json',
            'templates/bare/src/sagify_base/local_test/test_dir/input/data/training/'
        ]
    },
    setup_requires=['setuptools>=39.1.0'],
    extras_require={
        'ai-platform': ['future', 'google-cloud-storage', 'google-api-python-client'],
        'cli': ['click', 'docker', 'future'],
        'sagemaker': ['future', 'boto3']
    },
    entry_points="""
        [console_scripts]
        mldock=mldock.__main__:cli
    """,
    keywords=["docker", "machine learning", "ml", "ml services", "MLaaS"],
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6"
    ],
    python_requires='>=3.6',
)



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the 'License'). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the 'license' file accompanying this file. This file is
# distributed on an 'AS IS' BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
"""This module contains utilities related to dictionaries. These include
transforming dictionaries into environment variables, transforming into
command arguments, splitting dictionaries by key, and functionality from
the collections.Mapping abstract base class.
"""
from __future__ import absolute_import

import collections
import itertools
import json

import six

SplitResultSpec = collections.namedtuple("SplitResultSpec", "included excluded")


def to_env_vars(mapping):  # type: (dict) -> dict
    """Transform a dictionary in a dictionary of env vars.
     Example:
         >>>env_vars = mapping.to_env_vars({'model_dir': '/opt/ml/model', 'batch_size': 25})
         >>>
         >>>print(args)
         ['MODEL_DIR', '/opt/ml/model', 'BATCH_SIZE', 25]
     Args:
         mapping (dict[str, object]): A Python mapping.
     Returns:
         (dict): Dictionary of env vars.
     """

    def format_key(key):
        """Decode a key, adds a SM_ prefix to the key and upper case it."""
        if key:
            decoded_name = "SM_%s" % str(key).upper()
            return decoded_name
        else:
            return ""

    def format_value(_mapping):
        if six.PY3 and isinstance(_mapping, six.binary_type):
            # transforms a byte string (b'') in unicode
            return _mapping.decode("latin1")
        elif _mapping is None:
            return ""
        elif isinstance(_mapping, six.string_types):
            return str(_mapping)
        else:
            return json.dumps(_mapping, sort_keys=True, separators=(",", ":"), ensure_ascii=True)

    return {format_key(k): format_value(v) for k, v in mapping.items()}


def to_cmd_args(mapping):  # type: (dict) -> list
    """Transform a dictionary in a list of cmd arguments.
    Example:
        >>>args = mapping.to_cmd_args({'model_dir': '/opt/ml/model', 'batch_size': 25})
        >>>
        >>>print(args)
        ['--model_dir', '/opt/ml/model', '--batch_size', 25]
    Args:
        mapping (dict[str, object]): A Python mapping.
    Returns:
        (list): List of cmd arguments.
    """

    sorted_keys = sorted(mapping.keys())

    def arg_name(obj):
        string = _decode(obj)
        if string:
            return u"--%s" % string if len(string) > 1 else u"-%s" % string
        else:
            return u""

    arg_names = [arg_name(argument) for argument in sorted_keys]

    def arg_value(value):
        if hasattr(value, "items"):
            map_items = ["%s=%s" % (k, v) for k, v in sorted(value.items())]
            return ",".join(map_items)
        return _decode(value)

    arg_values = [arg_value(mapping[key]) for key in sorted_keys]

    items = zip(arg_names, arg_values)

    return [item for item in itertools.chain.from_iterable(items)]


def _decode(obj):  # type: (bytes or str or unicode or object) -> unicode # noqa ignore=F821
    """Decode an object to unicode.
    Args:
        obj (bytes or str or unicode or anything serializable): Object to be decoded.
    Returns:
        Object decoded in unicode.
    """
    if obj is None:
        return u""
    if six.PY3 and isinstance(obj, six.binary_type):
        # transforms a byte string (b'') in unicode
        return obj.decode("latin1")
    elif six.PY3:
        # PY3 strings are unicode.
        return str(obj)
    elif isinstance(obj, six.text_type):
        # returns itself if it is unicode
        return obj
    else:
        # decodes pY2 string to unicode
        return str(obj).decode("utf-8")


def split_by_criteria(
    dictionary, keys=None, prefix=None
):  # type: (dict, set or list or tuple) -> SplitResultSpec
    """Split a dictionary in two by the provided keys.
    Args:
        dictionary (dict[str, object]): A Python dictionary.
        keys (sequence [str]): A sequence of keys which will be added the split criteria.
        prefix (str): A prefix which will be added the split criteria.
    Returns:
        `SplitResultSpec` : A collections.namedtuple with the following attributes:
            * Args:
                included (dict[str, object]: A dictionary with the keys included in the criteria.
                excluded (dict[str, object]: A dictionary with the keys not included in the
                                             criteria.
    """
    keys = keys or []
    keys = set(keys)

    included_items = {
        k: dictionary[k]
        for k in dictionary.keys()
        if k in keys or (prefix and k.startswith(prefix))
    }
    excluded_items = {k: dictionary[k] for k in dictionary.keys() if k not in included_items}

    return SplitResultSpec(included=included_items, excluded=excluded_items)


class MappingMixin(collections.Mapping):
    """A mixin class that allows for the creation of a dictionary like object,
    with any built-in function that works with a dictionary. This is used by the
    environment._Env base class.
    """

    def properties(self):  # type: () -> list
        """
            Returns:
                (list[str]) List of public properties.
        """

        _type = type(self)
        return [_property for _property in dir(_type) if self._is_property(_property)]

    def _is_property(self, _property):
        return isinstance(getattr(type(self), _property), property)

    def __getitem__(self, k):
        """Built-in method override."""
        if not self._is_property(k):
            raise KeyError("Trying to access non property %s" % k)
        return getattr(self, k)

    def __len__(self):
        """Built-in method override."""
        return len(self.properties())

    def __iter__(self):
        """Built-in method override."""
        items = {_property: getattr(self, _property) for _property in self.properties()}
        return iter(items)

    def __str__(self):
        """Built-in method override."""
        return str(dict(self))



import os
from locative_ml_helpers.platform_helpers.gcp import storage

if __name__ == '__main__':

    print("Starting clean up")
    os.chdir('/home/')
    print("uploading model dir to storage")
    storage.package_and_upload_model_dir(local_path='/opt/ml/model', storage_dir_path=os.environ['SM_MODEL_DIR'], scheme='gs')

    print("uploading output data dir to storage")
    storage.package_and_upload_output_data_dir(local_path='/opt/ml/output', storage_dir_path=os.environ['SM_OUTPUT_DATA_DIR'], scheme='gs')
    print("Done")

